@@ -40,6 +40,7 @@
 #include "nautilus-trash-bar.h"
 #include "nautilus-x-content-bar.h"
 #include "nautilus-zoom-control.h"
+#include "nautilus-navigation-window-pane.h"
 #include <eel/eel-accessibility.h>
 #include <eel/eel-debug.h>
 #include <eel/eel-gdk-extensions.h>
@@ -48,11 +49,9 @@
 #include <eel/eel-gtk-macros.h>
 #include <eel/eel-stock-dialogs.h>
 #include <eel/eel-string.h>
-#include <eel/eel-mount-operation.h>
 #include <gtk/gtk.h>
 #include <gdk/gdkx.h>
 #include <glib/gi18n.h>
-#include <libgnomeui/gnome-icon-theme.h>
 #include <libnautilus-extension/nautilus-location-widget-provider.h>
 #include <libnautilus-private/nautilus-debug-log.h>
 #include <libnautilus-private/nautilus-file-attributes.h>
@@ -127,7 +126,7 @@ set_displayed_location (NautilusWindowSlot *slot, GFile *location)
         gboolean recreate;
 	char *name;
 
-	window = slot->window;
+	window = slot->pane->window;
         
         if (slot->current_location_bookmark == NULL || location == NULL) {
                 recreate = TRUE;
@@ -148,9 +147,6 @@ set_displayed_location (NautilusWindowSlot *slot, GFile *location)
                         : nautilus_bookmark_new (location, name);
 		g_free (name);
         }
-
-	nautilus_window_slot_update_title (slot);
-	nautilus_window_slot_update_icon (slot);
 }
 
 static void
@@ -296,25 +292,25 @@ handle_go_elsewhere (NautilusWindowSlot *slot, GFile *location)
 #endif
 }
 
-static void
-update_up_button (NautilusWindow *window)
+void
+nautilus_window_update_up_button (NautilusWindow *window)
 {
 	NautilusWindowSlot *slot;
-        gboolean allowed;
+	gboolean allowed;
 	GFile *parent;
 
-	slot = window->details->active_slot;
+	slot = window->details->active_pane->active_slot;
 
-        allowed = FALSE;
-        if (slot->location != NULL) {
+	allowed = FALSE;
+	if (slot->location != NULL) {
 		parent = g_file_get_parent (slot->location);
 		allowed = parent != NULL;
 		if (parent != NULL) {
 			g_object_unref (parent);
 		}
-        }
+	}
 
-        nautilus_window_allow_up (window, allowed);
+	nautilus_window_allow_up (window, allowed);
 }
 
 static void
@@ -325,9 +321,10 @@ viewed_file_changed_callback (NautilusFile *file,
         GFile *new_location;
 	gboolean is_in_trash, was_in_trash;
 
-	window = slot->window;
+	window = slot->pane->window;
 
         g_assert (NAUTILUS_IS_FILE (file));
+	g_assert (NAUTILUS_IS_WINDOW_PANE (slot->pane));
 	g_assert (NAUTILUS_IS_WINDOW (window));
 
 	g_assert (file == slot->viewed_file);
@@ -378,11 +375,11 @@ viewed_file_changed_callback (NautilusFile *file,
 					/* the path bar URI will be set to go_to_uri immediately
 					 * in begin_location_change, but we don't want the
 					 * inexistant children to show up anymore */
-					if (slot == window->details->active_slot) {
+					if (slot == slot->pane->active_slot) {
 						/* multiview-TODO also update NautilusWindowSlot
 						 * [which as of writing doesn't save/store any path bar state]
 						 */
-						nautilus_path_bar_clear_buttons (NAUTILUS_PATH_BAR (NAUTILUS_NAVIGATION_WINDOW (window)->path_bar));
+						nautilus_path_bar_clear_buttons (NAUTILUS_PATH_BAR (NAUTILUS_NAVIGATION_WINDOW_PANE (slot->pane)->path_bar));
 					}
 
 					nautilus_window_slot_go_to (slot, go_to_file, FALSE);
@@ -403,8 +400,8 @@ viewed_file_changed_callback (NautilusFile *file,
 				   slot->location)) {
                         g_object_unref (slot->location);
                         slot->location = new_location;
-			if (slot == window->details->active_slot) {
-				nautilus_window_sync_location_widgets (window);
+			if (slot == slot->pane->active_slot) {
+				nautilus_window_pane_sync_location_widgets (slot->pane);
 			}
                 } else {
 			/* TODO?
@@ -485,14 +482,16 @@ nautilus_window_slot_open_location_full (NautilusWindowSlot *slot,
 {
 	NautilusWindow *window;
         NautilusWindow *target_window;
+        NautilusWindowPane *pane;
         NautilusWindowSlot *target_slot;
 	NautilusWindowOpenFlags slot_flags;
         gboolean do_load_location = TRUE;
 	GFile *old_location;
 	char *old_uri, *new_uri;
 	int new_slot_position;
+	GList *l;
 
-	window = slot->window;
+	window = slot->pane->window;
 
         target_window = NULL;
 	target_slot = NULL;
@@ -590,7 +589,7 @@ nautilus_window_slot_open_location_full (NautilusWindowSlot *slot,
 			slot_flags = NAUTILUS_WINDOW_OPEN_SLOT_APPEND;
 		}
 
-		target_slot = nautilus_window_open_slot (window, slot_flags);
+		target_slot = nautilus_window_open_slot (window->details->active_pane, slot_flags);
 	}
 
         if ((flags & NAUTILUS_WINDOW_OPEN_FLAG_CLOSE_BEHIND) != 0) {
@@ -611,14 +610,16 @@ nautilus_window_slot_open_location_full (NautilusWindowSlot *slot,
 		if (target_window == window) {
 			target_slot = slot;
 		} else {
-			target_slot = target_window->details->active_slot;
+			target_slot = target_window->details->active_pane->active_slot;
 		}
 	}
 
         if ((!do_load_location) ||
 	    (target_window == window && target_slot == slot &&
 	     old_location && g_file_equal (old_location, location))) {
-		g_object_unref (old_location);
+		if (old_location) {
+			g_object_unref (old_location);
+		}
                 return;
         }
 	
@@ -628,6 +629,17 @@ nautilus_window_slot_open_location_full (NautilusWindowSlot *slot,
 
         begin_location_change (target_slot, location, new_selection,
                                NAUTILUS_LOCATION_CHANGE_STANDARD, 0, NULL);
+
+	/* Additionally, load this in all slots that have no location, this means
+	   we load both panes in e.g. a newly opened dual pane window. */
+	for (l = target_window->details->panes; l != NULL; l = l->next) {
+		pane = l->data;
+		slot = pane->active_slot;
+		if (slot->location == NULL && slot->pending_location == NULL) {
+			begin_location_change (slot, location, new_selection,
+					       NAUTILUS_LOCATION_CHANGE_STANDARD, 0, NULL);
+		}
+	}
 }
 
 void
@@ -724,7 +736,7 @@ report_current_content_view_failure_to_user (NautilusWindowSlot *slot)
 	NautilusWindow *window;
 	char *message;
 
-	window = slot->window;
+	window = slot->pane->window;
 
 	message = nautilus_window_slot_get_view_startup_error_label (slot);
   	eel_show_error_dialog (message,
@@ -740,7 +752,7 @@ report_nascent_content_view_failure_to_user (NautilusWindowSlot *slot,
 	NautilusWindow *window;
 	char *message;
 
-	window = slot->window;
+	window = slot->pane->window;
 
 	/* TODO? why are we using the current view's error label here, instead of the next view's?
  	 * This behavior has already been present in pre-slot days.
@@ -808,7 +820,7 @@ begin_location_change (NautilusWindowSlot *slot,
                   || type == NAUTILUS_LOCATION_CHANGE_FORWARD
                   || distance == 0);
 
-	window = slot->window;
+	window = slot->pane->window;
         g_assert (NAUTILUS_IS_WINDOW (window));
         g_object_ref (window);
 
@@ -870,8 +882,7 @@ begin_location_change (NautilusWindowSlot *slot,
 
 	nautilus_file_call_when_ready (slot->determine_view_file,
 				       NAUTILUS_FILE_ATTRIBUTE_INFO |
-				       NAUTILUS_FILE_ATTRIBUTE_MOUNT |
-				       NAUTILUS_FILE_ATTRIBUTE_METADATA,
+				       NAUTILUS_FILE_ATTRIBUTE_MOUNT,
                                        got_file_info_for_view_selection_callback,
 				       slot);
 
@@ -886,8 +897,9 @@ setup_new_spatial_window (NautilusWindowSlot *slot, NautilusFile *file)
 	char *geometry_string;
 	char *scroll_string;
 	gboolean maximized, sticky, above;
+	GtkAction *action;
 
-	window = slot->window;
+	window = slot->pane->window;
 
 	if (NAUTILUS_IS_SPATIAL_WINDOW (window) && !NAUTILUS_IS_DESKTOP_WINDOW (window)) {
 		/* load show hidden state */
@@ -896,15 +908,22 @@ setup_new_spatial_window (NautilusWindowSlot *slot, NautilusFile *file)
 			 NULL);
 		if (show_hidden_file_setting != NULL) {
 			if (strcmp (show_hidden_file_setting, "1") == 0) {
-				NAUTILUS_WINDOW (window)->details->show_hidden_files_mode = NAUTILUS_WINDOW_SHOW_HIDDEN_FILES_ENABLE;	
+				window->details->show_hidden_files_mode = NAUTILUS_WINDOW_SHOW_HIDDEN_FILES_ENABLE;
 			} else {
-				NAUTILUS_WINDOW (window)->details->show_hidden_files_mode = NAUTILUS_WINDOW_SHOW_HIDDEN_FILES_DISABLE;
+				window->details->show_hidden_files_mode = NAUTILUS_WINDOW_SHOW_HIDDEN_FILES_DISABLE;
 			}
+
+			/* Update the UI, since we initialize it to the default */
+			action = gtk_action_group_get_action (window->details->main_action_group, NAUTILUS_ACTION_SHOW_HIDDEN_FILES);
+			gtk_action_block_activate (action);
+			gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action),
+						      window->details->show_hidden_files_mode == NAUTILUS_WINDOW_SHOW_HIDDEN_FILES_ENABLE);
+			gtk_action_unblock_activate (action);
 		} else {
 			NAUTILUS_WINDOW (window)->details->show_hidden_files_mode = NAUTILUS_WINDOW_SHOW_HIDDEN_FILES_DEFAULT;
 		}
 		g_free (show_hidden_file_setting);
-		
+
 		/* load the saved window geometry */
 		maximized = nautilus_file_get_boolean_metadata
 			(file, NAUTILUS_METADATA_KEY_WINDOW_MAXIMIZED, FALSE);
@@ -978,7 +997,7 @@ mount_not_mounted_callback (GObject *source_object,
 
 	data = user_data;
 	slot = data->slot;
-	window = slot->window;
+	window = slot->pane->window;
 	cancellable = data->cancellable;
 	g_free (data);
 
@@ -1001,8 +1020,7 @@ mount_not_mounted_callback (GObject *source_object,
 	} else {
 		nautilus_file_invalidate_all_attributes (slot->determine_view_file);
 		nautilus_file_call_when_ready (slot->determine_view_file,
-					       NAUTILUS_FILE_ATTRIBUTE_INFO |
-					       NAUTILUS_FILE_ATTRIBUTE_METADATA,
+					       NAUTILUS_FILE_ATTRIBUTE_INFO,
 					       got_file_info_for_view_selection_callback,
 					       slot);
 	}
@@ -1028,7 +1046,7 @@ got_file_info_for_view_selection_callback (NautilusFile *file,
 	g_assert (NAUTILUS_IS_WINDOW_SLOT (slot));
 	g_assert (slot->determine_view_file == file);
 
-	window = slot->window;
+	window = slot->pane->window;
 	g_assert (NAUTILUS_IS_WINDOW (window));
 
 	slot->determine_view_file = NULL;
@@ -1043,7 +1061,7 @@ got_file_info_for_view_selection_callback (NautilusFile *file,
 	    !slot->tried_mount) {
 		slot->tried_mount = TRUE;
 
-		mount_op = eel_mount_operation_new (GTK_WINDOW (window));
+		mount_op = gtk_mount_operation_new (GTK_WINDOW (window));
 		location = nautilus_file_get_location (file);
 		data = g_new0 (MountNotMountedData, 1);
 		data->cancellable = g_cancellable_new ();
@@ -1073,7 +1091,7 @@ got_file_info_for_view_selection_callback (NautilusFile *file,
 		if (slot->location_change_type != NAUTILUS_LOCATION_CHANGE_FALLBACK) {
 			/* Look in metadata for view */
 			view_id = nautilus_file_get_metadata 
-				(file, NAUTILUS_METADATA_KEY_DEFAULT_COMPONENT, NULL);
+				(file, NAUTILUS_METADATA_KEY_DEFAULT_VIEW, NULL);
 			if (view_id != NULL && 
 			    !nautilus_view_factory_view_supports_uri (view_id,
 								      location,
@@ -1130,13 +1148,13 @@ got_file_info_for_view_selection_callback (NautilusFile *file,
 
 				if (!nautilus_is_root_directory (location)) {
 					if (!nautilus_is_home_directory (location)) {	
-						nautilus_window_slot_go_home (NAUTILUS_WINDOW (window)->details->active_slot, FALSE);
+						nautilus_window_slot_go_home (slot, FALSE);
 					} else {
 						GFile *root;
 
 						root = g_file_new_for_path ("/");
 						/* the last fallback is to go to a known place that can't be deleted! */
-						nautilus_window_slot_go_to (NAUTILUS_WINDOW (window)->details->active_slot, location, FALSE);
+						nautilus_window_slot_go_to (slot, location, FALSE);
 						g_object_unref (root);
 					}
 				} else {
@@ -1189,7 +1207,7 @@ create_content_view (NautilusWindowSlot *slot,
         NautilusView *view;
 	GList *selection;
 
-	window = slot->window;
+	window = slot->pane->window;
 
  	/* FIXME bugzilla.gnome.org 41243: 
 	 * We should use inheritance instead of these special cases
@@ -1264,7 +1282,7 @@ load_new_location (NautilusWindowSlot *slot,
 	g_assert (slot != NULL);
 	g_assert (location != NULL);
 
-	window = slot->window;
+	window = slot->pane->window;
 	g_assert (NAUTILUS_IS_WINDOW (window));
 
 	selection_copy = eel_g_object_list_copy (selection);
@@ -1345,7 +1363,7 @@ nautilus_window_report_location_change (NautilusWindow *window)
 
 	g_assert (NAUTILUS_IS_WINDOW (window));
 
-	slot = window->details->active_slot;
+	slot = window->details->active_pane->active_slot;
 	g_assert (NAUTILUS_IS_WINDOW_SLOT (slot));
 
 	location = NULL;
@@ -1371,7 +1389,7 @@ location_has_really_changed (NautilusWindowSlot *slot)
 	GtkWidget *widget;
 	GFile *location_copy;
 
-	window = slot->window;
+	window = slot->pane->window;
 
 	if (slot->new_content_view != NULL) {
 		widget = nautilus_view_get_widget (slot->new_content_view);
@@ -1421,7 +1439,7 @@ slot_add_extension_extra_widgets (NautilusWindowSlot *slot)
 		NautilusLocationWidgetProvider *provider;
 		
 		provider = NAUTILUS_LOCATION_WIDGET_PROVIDER (l->data);
-		widget = nautilus_location_widget_provider_get_widget (provider, uri, GTK_WIDGET (slot->window));
+		widget = nautilus_location_widget_provider_get_widget (provider, uri, GTK_WIDGET (slot->pane->window));
 		if (widget != NULL) {
 			nautilus_window_slot_add_extra_location_widget (slot, widget);
 		}
@@ -1446,6 +1464,11 @@ nautilus_window_slot_show_x_content_bar (NautilusWindowSlot *slot, GMount *mount
 			continue;
 		}
 
+		/* don't show the cluebar for windows software */
+		if (g_content_type_is_a (x_content_types[n], "x-content/win32-software")) {
+			continue;
+		}
+
 		/* only show the cluebar if a default app is available */
 		default_app = g_app_info_get_default_for_type (x_content_types[n], FALSE);
 		if (default_app != NULL)  {
@@ -1532,46 +1555,6 @@ found_mount_cb (GObject *source_object,
 	g_free (data);
 }
 
-void
-nautilus_window_sync_location_widgets (NautilusWindow *window)
-{
-	NautilusWindowSlot *slot;
-	char *uri;
-
-	slot = window->details->active_slot;
-
-	if (NAUTILUS_IS_NAVIGATION_WINDOW (window)) {
-		NautilusNavigationWindowSlot *navigation_slot;
-
-		navigation_slot = NAUTILUS_NAVIGATION_WINDOW_SLOT (slot);
-
-		/* Check if the back and forward buttons need enabling or disabling. */
-		update_up_button (window);
-		nautilus_navigation_window_allow_back (NAUTILUS_NAVIGATION_WINDOW (window),
-						       navigation_slot->back_list != NULL);
-		nautilus_navigation_window_allow_forward (NAUTILUS_NAVIGATION_WINDOW (window),
-							  navigation_slot->forward_list != NULL);
-
-		/* Change the location bar and path bar to match the current location. */
-		if (slot->location != NULL) {
-			/* this may be NULL if we just created the
- 			 * slot */
-			uri = nautilus_window_slot_get_location_uri (slot);
-			nautilus_navigation_bar_set_location (NAUTILUS_NAVIGATION_BAR (NAUTILUS_NAVIGATION_WINDOW (window)->navigation_bar),
-							      uri);
-			g_free (uri);
-			nautilus_path_bar_set_path (NAUTILUS_PATH_BAR (NAUTILUS_NAVIGATION_WINDOW (window)->path_bar),
-						    slot->location);
-		}
-	}
-
-	if (NAUTILUS_IS_SPATIAL_WINDOW (window)) {
-		/* Change the location button to match the current location. */
-		nautilus_spatial_window_set_location_button (NAUTILUS_SPATIAL_WINDOW (window),
-							     slot->location);
-	}
-}
-
 /* Handle the changes for the NautilusWindow itself. */
 static void
 update_for_new_location (NautilusWindowSlot *slot)
@@ -1583,7 +1566,7 @@ update_for_new_location (NautilusWindowSlot *slot)
 	gboolean location_really_changed;
 	FindMountData *data;
 
-	window = slot->window;
+	window = slot->pane->window;
 
 	new_location = slot->pending_location;
 	slot->pending_location = NULL;
@@ -1615,13 +1598,12 @@ update_for_new_location (NautilusWindowSlot *slot)
 				 G_CALLBACK (viewed_file_changed_callback), slot, 0);
         nautilus_file_unref (file);
 
-	if (slot == window->details->active_slot) {
+	if (slot == window->details->active_pane->active_slot) {
 		/* Check if we can go up. */
-		update_up_button (window);
-	}
+		nautilus_window_update_up_button (window);
 
-	if (slot == window->details->active_slot) {
 		nautilus_window_sync_zoom_widgets (window);
+
 		/* Set up the content view menu for this new location. */
 		nautilus_window_load_view_as_menus (window);
 
@@ -1663,14 +1645,18 @@ update_for_new_location (NautilusWindowSlot *slot)
 		slot_add_extension_extra_widgets (slot);
 	}
 
-	if (slot == window->details->active_slot) {
-		nautilus_window_sync_location_widgets (window);
+	nautilus_window_slot_update_title (slot);
+	nautilus_window_slot_update_icon (slot);
+
+	if (slot == slot->pane->active_slot) {
+		nautilus_window_pane_sync_location_widgets (slot->pane);
 
 		if (location_really_changed) {
-			nautilus_window_sync_search_widgets (window);
+			nautilus_window_pane_sync_search_widgets (slot->pane);
 		}
 
-		if (NAUTILUS_IS_NAVIGATION_WINDOW (window)) {
+		if (NAUTILUS_IS_NAVIGATION_WINDOW (window) &&
+		    slot->pane == window->details->active_pane) {
 			nautilus_navigation_window_load_extension_toolbar_items (NAUTILUS_NAVIGATION_WINDOW (window));
 		}
 	}
@@ -1710,7 +1696,7 @@ end_location_change (NautilusWindowSlot *slot)
 	NautilusWindow *window;
 	char *uri;
 
-	window = slot->window;
+	window = slot->pane->window;
 
 	uri = nautilus_window_slot_get_location_uri (slot);
 	if (uri) {
@@ -1735,7 +1721,7 @@ free_location_change (NautilusWindowSlot *slot)
 {
 	NautilusWindow *window;
 
-	window = slot->window;
+	window = slot->pane->window;
 	g_assert (NAUTILUS_IS_WINDOW (window));
 
 	if (slot->pending_location) {
@@ -1906,7 +1892,7 @@ display_view_selection_failure (NautilusWindow *window, NautilusFile *file,
 				detail_message = g_strdup_printf (_("Nautilus cannot handle \"%s\" locations."),
 								  scheme_string);
 			} else {
-				detail_message = g_strdup (_("Nautilus cannot handle this kind of locations."));
+				detail_message = g_strdup (_("Nautilus cannot handle this kind of location."));
 			}
 			g_free (scheme_string);
 			break;
@@ -1960,7 +1946,7 @@ nautilus_window_slot_stop_loading (NautilusWindowSlot *slot)
 {
 	NautilusWindow *window;
 
-	window = NAUTILUS_WINDOW (slot->window);
+	window = NAUTILUS_WINDOW (slot->pane->window);
 	g_assert (NAUTILUS_IS_WINDOW (window));
 
 	nautilus_view_stop_loading (slot->content_view);
@@ -1986,7 +1972,7 @@ nautilus_window_slot_set_content_view (NautilusWindowSlot *slot,
 	g_assert (slot->location != NULL);
 	g_assert (id != NULL);
 
-	window = slot->window;
+	window = slot->pane->window;
 	g_assert (NAUTILUS_IS_WINDOW (window));
   
 	uri = nautilus_window_slot_get_location_uri (slot);
@@ -2003,7 +1989,7 @@ nautilus_window_slot_set_content_view (NautilusWindowSlot *slot,
 
 	file = nautilus_file_get (slot->location);
 	nautilus_file_set_metadata 
-		(file, NAUTILUS_METADATA_KEY_DEFAULT_COMPONENT, NULL, id);
+		(file, NAUTILUS_METADATA_KEY_DEFAULT_VIEW, NULL, id);
         nautilus_file_unref (file);
         
         nautilus_window_slot_set_allow_stop (slot, TRUE);
@@ -2019,7 +2005,7 @@ nautilus_window_slot_set_content_view (NautilusWindowSlot *slot,
 }
 
 void
-nautilus_window_manage_views_close_slot (NautilusWindow *window,
+nautilus_window_manage_views_close_slot (NautilusWindowPane *pane,
 					 NautilusWindowSlot *slot)
 {
 	if (slot->content_view != NULL) {
@@ -2041,7 +2027,7 @@ nautilus_navigation_window_back_or_forward (NautilusNavigationWindow *window,
         guint len;
         NautilusBookmark *bookmark;
 
-	slot = NAUTILUS_WINDOW (window)->details->active_slot;
+	slot = NAUTILUS_WINDOW (window)->details->active_pane->active_slot;
 	navigation_slot = (NautilusNavigationWindowSlot *) slot;
 	list = back ? navigation_slot->back_list : navigation_slot->forward_list;
 
@@ -2118,6 +2104,6 @@ nautilus_window_reload (NautilusWindow *window)
 {
 	g_assert (NAUTILUS_IS_WINDOW (window));
 
-	nautilus_window_slot_reload (window->details->active_slot);
+	nautilus_window_slot_reload (window->details->active_pane->active_slot);
 }
 
